@@ -47,12 +47,9 @@
 #include "kis_tiff_stream.h"
 #include "kis_tiff_writer_visitor.h"
 
-#include <kis_ycbcr_u8_colorspace.h> // TODO: in the future YCbCrU8/16 will be moved in a plugins and we won't need anymore that cludge
-#include <kis_ycbcr_u16_colorspace.h>
-
 namespace {
 
-    QString getColorSpaceForColorType(uint16 color_type, uint16 color_nb_bits, TIFF *image, uint16 &nbchannels, uint16 &extrasamplescount, uint8 &destDepth) {
+    QString getColorSpaceForColorType(uint16 color_type, uint16 color_nb_bits, TIFF *image, uint16 &nbchannels, uint16 &extrasamplescount, uint8 &destDepth, uint16 sampletype) {
         if(color_type == PHOTOMETRIC_MINISWHITE || color_type == PHOTOMETRIC_MINISBLACK)
         {
             if(nbchannels == 0) nbchannels = 1;
@@ -68,13 +65,26 @@ namespace {
         } else if(color_type == PHOTOMETRIC_RGB  /*|| color_type == */ ) {
             if(nbchannels == 0) nbchannels = 3;
             extrasamplescount = nbchannels - 3; // FIX the extrasamples count in case of
-            if(color_nb_bits <= 8)
+            if(sampletype == SAMPLEFORMAT_IEEEFP)
             {
-                destDepth = 8;
-                return "RGBA";
-            } else {
+              if(color_nb_bits == 16)
+              {
                 destDepth = 16;
-                return "RGBA16";
+                return "RGBAF16HALF";
+              } else if( color_nb_bits == 32) {
+                destDepth = 32;
+                return "RGBAF32";
+              }
+              return "";
+            } else {
+              if(color_nb_bits <= 8)
+              {
+                  destDepth = 8;
+                  return "RGBA";
+              } else {
+                  destDepth = 16;
+                  return "RGBA16";
+              }
             }
         } else if(color_type == PHOTOMETRIC_YCBCR ) {
             if(nbchannels == 0) nbchannels = 3;
@@ -124,7 +134,11 @@ namespace {
                 destDepth = 16;
                 return "CMYKA16";
             }
-        } else if(color_type == PHOTOMETRIC_CIELAB || color_type == PHOTOMETRIC_ICCLAB ) {
+        } else if(color_type == PHOTOMETRIC_CIELAB
+#ifdef PHOTOMETRIC_ICCLAB
+	 || color_type == PHOTOMETRIC_ICCLAB
+#endif
+	 ) {
             destDepth = 16;
             if(nbchannels == 0) nbchannels = 3;
             extrasamplescount = nbchannels - 3; // FIX the extrasamples count in case of
@@ -159,7 +173,7 @@ KisImageBuilder_Result KisTIFFConverter::decode(const KURL& uri)
     TIFF *image = 0;
     if((image = TIFFOpen(QFile::encodeName(uri.path()), "r")) == NULL){
         kdDebug(41008) << "Could not open the file, either it doesn't exist, either it is not a TIFF : " << uri.path() << endl;
-        if (image) TIFFClose(image);
+
         return (KisImageBuilder_RESULT_BAD_FETCH);
     }
     do {
@@ -193,6 +207,11 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
         kdDebug(41008) <<  "Image does not define its depth" << endl;
         depth = 1;
     }
+    uint16 sampletype;
+    if((TIFFGetField(image, TIFFTAG_SAMPLEFORMAT, &sampletype) == 0)){
+        kdDebug(41008) <<  "Image does not define its sample type" << endl;
+        sampletype =  SAMPLEFORMAT_UINT;
+    }
     // Determine the number of channels (usefull to know if a file has an alpha or not
     uint16 nbchannels;
     if(TIFFGetField(image, TIFFTAG_SAMPLESPERPIXEL, &nbchannels) == 0){
@@ -212,7 +231,7 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
         color_type = PHOTOMETRIC_MINISWHITE;
     }
     uint8 dstDepth;
-    QString csName = getColorSpaceForColorType(color_type, depth, image, nbchannels, extrasamplescount, dstDepth);
+    QString csName = getColorSpaceForColorType(color_type, depth, image, nbchannels, extrasamplescount, dstDepth,sampletype);
     if(csName.isEmpty()) {
         kdDebug(41008) << "Image has an unsupported colorspace : " << color_type << " for this depth : "<< depth << endl;
         TIFFClose(image);
@@ -238,23 +257,7 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
     
     // Retrieve a pointer to the colorspace
     KisColorSpace* cs = 0;
-    if( csName == "YCbCrAU8" ) { // TODO: in the future YCbCrU8/16 will be moved in a plugins and we won't need anymore that cludge
-        if (profile && profile->isSuitableForOutput())
-        {
-            kdDebug(41008) << "image has embedded profile: " << profile -> productName() << "\n";
-            cs = new KisYCbCrU8ColorSpace( KisMetaRegistry::instance()->csRegistry(), profile);
-        } else {
-            cs = new KisYCbCrU8ColorSpace( KisMetaRegistry::instance()->csRegistry(), 0);
-        }
-    } else if( csName == "YCbCrAU16") {
-        if (profile && profile->isSuitableForOutput())
-        {
-            kdDebug(41008) << "image has embedded profile: " << profile -> productName() << "\n";
-            cs = new KisYCbCrU16ColorSpace( KisMetaRegistry::instance()->csRegistry(), profile);
-        } else {
-            cs = new KisYCbCrU16ColorSpace( KisMetaRegistry::instance()->csRegistry(), 0);
-        }
-    } else if (profile && profile->isSuitableForOutput())
+    if (profile && profile->isSuitableForOutput())
     {
         kdDebug(41008) << "image has embedded profile: " << profile -> productName() << "\n";
         cs = KisMetaRegistry::instance()->csRegistry()->getColorSpace(csName, profile);
@@ -376,12 +379,14 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
             postprocessor = new KisTIFFPostProcessor(nbcolorsamples);
         }
         break;
+#ifdef PHOTOMETRIC_ICCLAB
         case PHOTOMETRIC_ICCLAB:
         {
             poses[0] = 0; poses[1] = 1; poses[2] = 2; poses[3] = 3;
             postprocessor = new KisTIFFPostProcessorICCLABtoCIELAB(nbcolorsamples);
         }
         break;
+#endif
         case PHOTOMETRIC_RGB:
         {
             poses[0] = 2; poses[1] = 1; poses[2] = 0; poses[3] = 3;
@@ -438,6 +443,8 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
         tiffReader = new KisTIFFReaderTarget8bit( layer->paintDevice(), poses, alphapos, depth, nbcolorsamples, extrasamplescount, transform, postprocessor);
     } else if(dstDepth == 16) {
         tiffReader = new KisTIFFReaderTarget16bit( layer->paintDevice(), poses, alphapos, depth, nbcolorsamples, extrasamplescount, transform, postprocessor);
+    } else if(dstDepth == 32) {
+        tiffReader = new KisTIFFReaderTarget32bit( layer->paintDevice(), poses, alphapos, depth, nbcolorsamples, extrasamplescount, transform, postprocessor);
     }
     
     if(TIFFIsTiled(image))
@@ -568,15 +575,6 @@ KisImageBuilder_Result KisTIFFConverter::readTIFFDirectory( TIFF* image)
         delete[] ps_buf;
     }
     
-    // Convert YCbCr to RGB
-    if( csName == "YCbCrAU8" ) { // TODO: in the future YCbCrU8/16 will be moved in a plugins and we won't need anymore that cludge
-        KisColorSpace * dstCS = KisMetaRegistry::instance()->csRegistry()->getColorSpace(KisID("RGBA", ""), "");
-        m_img->convertTo(dstCS);
-    } else if( csName == "YCbCrAU16") {
-        KisColorSpace * dstCS = KisMetaRegistry::instance()->csRegistry()->getColorSpace(KisID("RGBA16", ""), "");
-        m_img->convertTo(dstCS);
-    }
-
     m_img->addLayer(layer, m_img->rootLayer(), 0);
     return KisImageBuilder_RESULT_OK;
 }
