@@ -1,5 +1,5 @@
-/* Sidewinder - Portable library for spreadsheet
-   Copyright (C) 2003 Ariya Hidayat <ariya@kde.org>
+/* Swinder - Portable library for spreadsheet
+   Copyright (C) 2003-2006 Ariya Hidayat <ariya@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,19 +13,20 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, US
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA
 */
 
-#ifndef SIDEWINDER_EXCEL_H
-#define SIDEWINDER_EXCEL_H
+#ifndef SWINDER_EXCEL_H
+#define SWINDER_EXCEL_H
 
 #include <string>
 #include <iostream>
+#include <vector>
 
-#include "sidewinder.h"
+#include "swinder.h"
 
-namespace Sidewinder
+namespace Swinder
 {
 
 /**
@@ -66,9 +67,7 @@ public:
   void setSize( unsigned size ); // HACKS
   
   
-  static EString fromUnicodeString( const void* p, unsigned maxsize = 0 );
-  
-  static EString fromByteString( const void* p, unsigned maxsize = 0 );
+  static EString fromUnicodeString( const void* p, bool longString, unsigned maxsize = 0 );
   
   static EString fromSheetName( const void* p, unsigned maxsize = 0 );
   
@@ -82,6 +81,121 @@ private:
   Private* d;
 };
 
+class FormulaToken
+{
+public:
+
+  enum
+  {
+    // should match Excel's PTG
+    Unused      = 0x00,
+    Matrix      = 0x01,
+    Table       = 0x02,
+    Add         = 0x03,
+    Sub         = 0x04,
+    Mul         = 0x05,
+    Div         = 0x06,
+    Power       = 0x07,
+    Concat      = 0x08,
+    LT          = 0x09,
+    LE          = 0x0a,
+    EQ          = 0x0b,
+    GE          = 0x0c,
+    GT          = 0x0d,
+    NE          = 0x0e,
+    Intersect   = 0x0f,
+    List        = 0x10,
+    Range       = 0x11,
+    UPlus       = 0x12,
+    UMinus      = 0x13,
+    Percent     = 0x14,
+    Paren       = 0x15,
+    MissArg     = 0x16,
+    String      = 0x17,
+    NatFormula  = 0x18,
+    Attr        = 0x19,
+    Sheet       = 0x1a,
+    EndSheet    = 0x1b,
+    ErrorCode   = 0x1c,
+    Bool        = 0x1d,
+    Integer     = 0x1e,
+    Float       = 0x1f,
+    Array       = 0x20,
+    Function    = 0x21,
+    FunctionVar = 0x22,
+    Name        = 0x23,
+    Ref         = 0x24,
+    Area        = 0x25,
+    MemArea     = 0x26,
+    MemErr      = 0x27,
+    MemNoMem    = 0x28,
+    MemFunc     = 0x29,
+    RefErr      = 0x2a,
+    AreaErr     = 0x2b,
+    RefN        = 0x2c,
+    AreaN       = 0x2d,
+    MemAreaN    = 0x2e,
+    MemNoMemN   = 0x2f,
+    NameX       = 0x39,
+    Ref3d       = 0x3a,
+    Area3d      = 0x3b,
+    RefErr3d    = 0x3c,
+    AreaErr3d   = 0x3d
+  };
+
+  FormulaToken();
+  FormulaToken( unsigned id );
+  FormulaToken( const FormulaToken& );
+  ~FormulaToken();
+  
+  // token id, excluding token class
+  unsigned id() const;
+  const char* idAsString() const;
+  
+  // Excel version
+  unsigned version() const;
+  void setVersion( unsigned version );  // Excel version
+  
+  // size of data, EXCLUDING the byte for token id
+  unsigned size() const;
+  void setData( unsigned size, const unsigned char* data );
+  
+  // only when id returns ErrorCode, Bool, Integer, Float, or String
+  Value value() const;
+  
+  // only when id is Function or FunctionVar
+  unsigned functionIndex() const;
+  const char* functionName() const;  // for non external function
+  unsigned functionParams() const;
+  
+  // only when id is Ref or Ref3d
+  UString ref( unsigned row, unsigned col ) const;
+
+  // only when id is Area or Area3d
+  UString area( unsigned row, unsigned col ) const;
+
+  // only when id is Ref3d or Area3d
+  unsigned externSheetRef() const;
+
+  // only when id is Attr
+  unsigned attr() const;
+
+  // only when id is NameX
+  unsigned nameIndex() const;
+
+  // only when id is Matrix
+  unsigned refRow() const;
+  unsigned refColumn() const;
+
+private:
+  class Private;
+  Private *d;  
+};
+
+typedef std::vector<FormulaToken> FormulaTokens;
+
+std::ostream& operator<<( std::ostream& s, FormulaToken token );
+
 /**
   Class Record represents a base class for all other type record,
   hence do not use this class in real life.
@@ -97,6 +211,10 @@ public:
   */  
   static const unsigned int id;
   
+  virtual unsigned int rtti(){
+	  return this->id;
+  }
+  
   /**
     Creates a new generic record.
   */  
@@ -292,6 +410,8 @@ private:
 };
 
 /**
+  \brief Backup upon save.
+  
   Class BackupRecord represents Backup record, which determines whether 
   Microsoft Excel makes a backup of the file while saving.
  */
@@ -311,6 +431,9 @@ public:
    */
   ~BackupRecord();
   
+  unsigned int rtti(){
+	  return this->id;
+  }
   /**
    * Returns true if a backup is made when saving the file.
    * 
@@ -342,6 +465,8 @@ private:
 
 
 /**
+  \brief Beginning of file/set of records.
+  
   Class BOFRecord represents BOF (Beginning of File) record, which
   is used to mark beginning of a set of records (following the BOF record).
   For each BOF record, there should also be one corresponding EOF record.
@@ -361,10 +486,14 @@ public:
   */
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
     Supported BOF type.
   */
-  enum { UnknownType = 0, Workbook, Worksheet, Chart, VBModule };
+  enum { UnknownType = 0, Workbook, Worksheet, Chart, VBModule, MacroSheet, Workspace };
 
   /**
    * Creates a new BOF record.
@@ -415,6 +544,8 @@ private:
 };
 
 /**
+  \brief Blank cell.
+  
   Class BlankRecord represents a blank cell. It contains information
   about cell address and formatting.
  */
@@ -424,6 +555,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Blank record.
    */
@@ -442,6 +577,8 @@ private:
 };
 
 /**
+  \brief Boolean value or error code.
+  
   Class BOFRecord represents BoolErr record, which
   is used to store boolean value or error code of a cell.
  */
@@ -454,14 +591,7 @@ public:
   */
   static const unsigned int id;
 
-  enum { ErrorUnknown, 
-   ErrorNull, 
-   ErrorDivZero, 
-   ErrorValue, 
-   ErrorRef, 
-   ErrorName,
-   ErrorNum, 
-   ErrorNA };
+  unsigned int rtti(){ return this->id; }
 
   /**
    * Creates a new BoolErr record.
@@ -474,32 +604,9 @@ public:
   virtual ~BoolErrRecord();
 
   /**
-   * Returns true if the record specifies boolean value.
+   * Returns value of the cell, could be either boolean or error.
    */
-  bool isBool() const;
-
-  /**
-   * Returns true if the record specifies error code.
-   */
-  bool isError() const;
-
-  /**
-   * Returns boolean value of the cell. It is valid only when isBool()
-   * returns true.
-   */
-  bool value() const;
-
-  /**
-   * Returns error code of the cell. It is valid only when isError()
-   * returns true.
-   */
-  unsigned errorCode() const;
-  
-  /**
-   * Returns error code of the cell as string. It is valid only when isError()
-   * returns true.
-   */
-  const char* errorCodeAsString() const;
+  Value value() const;
 
   virtual void setData( unsigned size, const unsigned char* data );
 
@@ -517,8 +624,10 @@ private:
 };
 
 /**
-  Class BottomMarginRecord holds information about bottom margin.
+  \brief Bottom margin.
   
+  Class BottomMarginRecord holds information about bottom margin 
+  (in inches).
  */
 class BottomMarginRecord : public Record
 {
@@ -526,6 +635,9 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
   /**
    * Creates a new BottomMargin record.
    */
@@ -562,6 +674,8 @@ private:
 };
 
 /**
+  \brief Sheet information.
+  
   Class BoundSheetRecord represents BoundSheet record, which defines a sheet
   within the workbook. There must be exactly one BoundSheet record for 
   each sheet.
@@ -580,6 +694,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new BoundSheet record.
    */
@@ -660,6 +778,8 @@ private:
 };
 
 /**
+  \brief Automatic recalculation mode.
+  
   Class CalcModeRecord represents CalcMode record, which specifies whether 
   to (re)calculate formulas manually or automatically.
  */
@@ -669,6 +789,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new CalcMode record.
    */
@@ -709,6 +833,8 @@ private:
 };
 
 /**
+  \brief Columns width and format.
+  
   Class ColInfoRecord represents ColInfo record, which provides information
   (such as column width and formatting) for a span of columns.
  */
@@ -718,6 +844,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new ColInfo record.
    */
@@ -820,6 +950,8 @@ private:
 };
 
 /**
+  \brief Date reference.
+  
   Class DateModeRecord represents DateMode record, which specifies
   reference date for displaying date value of given serial number.
   If base1904 is true, the reference date is 1st of January, 1904 (in which
@@ -833,6 +965,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new DateMode record.
    */
@@ -875,8 +1011,10 @@ private:
 };
 
 /**
-  Class EOFRecord represents Dimension record, which contains the range address 
-  of the used area in the current sheet.
+  \brief Range of used area.
+  
+  Class DimensionRecord represents Dimension record, which contains the 
+  range address of the used area in the current sheet.
  */
 class DimensionRecord : public Record
 {
@@ -884,6 +1022,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Dimension record.
    */
@@ -965,7 +1107,88 @@ private:
    Private *d;
 };
 
+class ExternNameRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  unsigned int rtti(){
+    return this->id;
+  }
+
+  ExternNameRecord();
+
+  ~ExternNameRecord();
+
+  // one-based sheet index
+  // if 0 means AddIn function
+  unsigned sheetIndex() const;
+
+  void setSheetIndex( unsigned sheetIndex );
+
+  UString externName() const;
+
+  void setExternName( const UString& name );
+
+  virtual void setData( unsigned size, const unsigned char* data );
+
+
+
+  virtual const char* name(){ return "EXTERNNAME"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+   // no copy or assign
+   ExternNameRecord( const ExternNameRecord& );
+   ExternNameRecord& operator=( const ExternNameRecord& );
+   
+   class Private;
+   Private *d;
+};
+
+class ExternSheetRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  unsigned int rtti(){
+    return this->id;
+  }
+
+  ExternSheetRecord();
+
+  ~ExternSheetRecord();
+
+  unsigned count() const;
+
+  unsigned refIndex(unsigned i) const;
+  unsigned firstSheet(unsigned i) const;
+  unsigned lastSheet(unsigned i) const;
+
+  UString refName() const;
+
+  virtual void setData( unsigned size, const unsigned char* data );
+
+  virtual const char* name(){ return "EXTERNSHEET"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+   // no copy or assign
+   ExternSheetRecord( const ExternNameRecord& );
+   ExternSheetRecord& operator=( const ExternNameRecord& );
+   
+   class Private;
+   Private *d;
+};
+
+
 /**
+  \brief End of record set.
+  
   Class EOFRecord represents EOF record, which marks the end of records
   for a specific object. EOF record should be always in pair with BOF Record.
 
@@ -978,6 +1201,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new EOF record.
    */
@@ -1000,7 +1227,41 @@ private:
    EOFRecord& operator=( const EOFRecord& );
 };
 
+class FilepassRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  unsigned int rtti(){
+    return this->id;
+  }
+
+  /**
+   * Creates a new FILEPASS record.
+   */
+  FilepassRecord();
+
+  /**
+   * Destroy the record.
+   */
+  virtual ~FilepassRecord();
+  
+  virtual void setData( unsigned size, const unsigned char* data );
+
+  virtual const char* name(){ return "FILEPASS"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+   // no copy or assign
+   FilepassRecord( const FilepassRecord& );
+   FilepassRecord& operator=( const FilepassRecord& );
+};
+
 /**
+  \brief Font information.
+  
   Class FontRecord represents Font record, which has the information
   about specific font used in the document. Several Font records creates
   a font table, whose index will be referred in XFormat records.
@@ -1018,6 +1279,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Font record.
    */
@@ -1047,8 +1312,8 @@ public:
     None = 0, 
     Single = 1, 
     Double = 2, 
-    SingleAccounting = 3, 
-    DoubleAccounting = 4 };
+    SingleAccounting = 0x21, 
+    DoubleAccounting = 0x22 };
   
   unsigned height() const;
   void setHeight( unsigned h );
@@ -1137,17 +1402,17 @@ public:
    * Returns Font::Superscript if superscript is set, or Font::Subscript
    * if subscript is set, or Font::Normal in other case.
    *
-   * \sa setScript
+   * \sa setEscapement
    */
-  unsigned script() const;
+  unsigned escapement() const;
   
   /**
    * Sets the superscript or subscript. If s is Font::Superscript, then
    * superscript is set. If s is Font::Subscript, then subscript is set.
    *
-   * \sa script
+   * \sa escapement
    */
-  void setScript( unsigned s );
+  void setEscapement( unsigned s );
   
   /**
    * Returns the underline style of the font. Possible values are
@@ -1179,7 +1444,9 @@ private:
 };
 
 /**
-  Class FooterRecord holds information about sheet header.
+  \brief Sheet footer.
+  
+  Class FooterRecord holds information about sheet footer.
   
  */
 class FooterRecord : public Record
@@ -1188,8 +1455,12 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
-   * Creates a new Header record.
+   * Creates a new Footer record.
    */
   FooterRecord();
   
@@ -1224,6 +1495,8 @@ private:
 };
 
 /**
+  \brief Number formatting string.
+  
   Class FormatRecord contains information about a number format. 
   All Format records occur together in a sequential list.
   An XFRecord might refer to the specific Format record using
@@ -1238,6 +1511,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Format record.
    */
@@ -1293,12 +1570,16 @@ public:
   
   virtual void setData( unsigned size, const unsigned char* data );
 
+  virtual void dump( std::ostream& out ) const;
+
 private:
    class Private;
    Private *d;
 };
 
 /**
+  \brief Formula.
+  
   Class FormulaRecord holds Formula record.
   
  */
@@ -1307,9 +1588,14 @@ class FormulaRecord : public Record, public CellInfo
 public:
 
   static const unsigned int id;
+  static const unsigned int idOld; // for Pocket Excel support
+
+  unsigned int rtti(){
+	  return this->id;
+  }
 
   /**
-   * Creates a new Header record.
+   * Creates a new formula record.
    */
   FormulaRecord();
   
@@ -1328,6 +1614,8 @@ public:
    */
   void setResult( const Value& v );
   
+  FormulaTokens tokens() const;
+  
   virtual void setData( unsigned size, const unsigned char* data );
 
   virtual const char* name(){ return "FORMULA"; }
@@ -1344,15 +1632,21 @@ private:
 };
 
 /**
-  Class HeaderRecord holds information about sheet header.
+  \brief Sheet header.
   
+  Class HeaderRecord holds information about sheet header.  
  */
+
 class HeaderRecord : public Record
 {
 public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Header record.
    */
@@ -1404,6 +1698,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Label record.
    */
@@ -1454,6 +1752,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new LabelSST record.
    */
@@ -1495,6 +1797,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new LeftMargin record.
    */
@@ -1544,6 +1850,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new MergedCells record.
    */
@@ -1607,6 +1917,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new MulBlank record.
    */
@@ -1655,6 +1969,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new MulRK record.
    */
@@ -1695,6 +2013,8 @@ public:
    */
   double asFloat( unsigned i ) const;
   
+  unsigned encodedRK( unsigned i ) const;
+  
   virtual const char* name(){ return "MULRK"; }
 
   virtual void dump( std::ostream& out ) const;
@@ -1713,6 +2033,41 @@ private:
    virtual unsigned xfIndex() const { return CellInfo::xfIndex(); }
 };
 
+
+class NameRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  unsigned int rtti(){
+    return this->id;
+  }
+
+  NameRecord();
+
+  ~NameRecord();
+
+  UString definedName() const;
+
+  void setDefinedName( const UString& name );
+
+  virtual void setData( unsigned size, const unsigned char* data );
+
+  virtual const char* name(){ return "NAME"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+   // no copy or assign
+   NameRecord( const NameRecord& );
+   NameRecord& operator=( const NameRecord& );
+   
+   class Private;
+   Private *d;
+};
+
+
 /**
   Class NumberRecord represents a cell that contains a floating point value. 
 
@@ -1723,6 +2078,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Number record.
    */
@@ -1772,6 +2131,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Palette record.
    */
@@ -1819,6 +2182,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new RightMargin record.
    */
@@ -1867,6 +2234,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new RK record.
    */
@@ -1923,6 +2294,8 @@ public:
    */
   void setFloat( double f );
 
+  unsigned encodedRK() const;
+  
   virtual const char* name(){ return "RK"; }
   
   virtual void dump( std::ostream& out ) const;
@@ -1946,6 +2319,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Row record.
    */
@@ -2044,6 +2421,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new Label record.
    */
@@ -2097,6 +2478,10 @@ public:
 
   static const unsigned int id;
   
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new SST record.
    */ 
@@ -2135,6 +2520,115 @@ private:
 };
 
 
+/**
+  Class String represents a string record, which holds the value of 
+  calculation if a formula returns a string. This record must appear 
+  right after the associated formula record.
+
+  \sa FormulaRecord
+  
+ */
+class StringRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+  
+  unsigned int rtti(){ return this->id; }
+
+  /**
+   * Creates a new string record.
+   */ 
+  StringRecord();
+  
+  /**
+   * Destroys the record.
+   */ 
+  virtual ~StringRecord();
+
+  virtual void setData( unsigned size, const unsigned char* data );
+  
+  /**
+    Returns the string (in Unicode).
+   */  
+  UString ustring() const;
+  
+  /**
+    Returns the string as a value.
+   */  
+  Value value() const;
+  
+  virtual const char* name(){ return "STRING"; }
+  
+  virtual void dump( std::ostream& out ) const;
+  
+private:
+   // no copy or assign
+   StringRecord( const SSTRecord& );
+   StringRecord& operator=( const SSTRecord& );
+
+   class Private;
+   Private *d;
+};
+
+/**
+  Class SupbookRecord stores references to external workbook.
+  
+ */
+class SupbookRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  typedef enum 
+  {
+	  UnknownRef,
+	  ExternalRef,
+	  InternalRef,
+	  AddInRef,
+	  ObjectLink
+  } ReferenceType;
+
+  unsigned int rtti(){
+	  return this->id;
+  }
+
+  /**
+   * Creates a new Supbook record.
+   */
+  SupbookRecord();
+
+  /**
+   * Destroy the record.
+   */
+  ~SupbookRecord();
+
+  /**
+   * Returns the type of the reference.
+   */
+  ReferenceType referenceType() const;
+
+  /**
+   * Sets the type of the reference.
+   */
+  void setReferenceType(ReferenceType type);
+
+  virtual void setData( unsigned size, const unsigned char* data );
+
+  virtual const char* name(){ return "SUPBOOK"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+  // no copy or assign
+  SupbookRecord( const SupbookRecord& );
+  SupbookRecord& operator=( const SupbookRecord& );
+
+  class Private;
+  Private* d;
+};
+
 /**
   Class TopMarginRecord holds information about top margin.
   
@@ -2145,6 +2639,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new TopMargin record.
    */
@@ -2192,6 +2690,10 @@ public:
 
   static const unsigned int id;
 
+  unsigned int rtti(){
+	  return this->id;
+  }
+
   /**
    * Creates a new XF record.
    */
@@ -2673,19 +3175,20 @@ private:
   Private* d;
 };
 
+typedef std::vector<UString> UStringStack;
 
-class ExcelReader: public Reader
+
+class ExcelReader
 {
 public:
   ExcelReader();
   virtual ~ExcelReader();
+  bool load( Workbook* workbook, const char* filename );
   
-  virtual Workbook* load( const char* filename );
-    
-private:
-
-  void handleRecord( Record* record );
+protected:  
+  virtual void handleRecord( Record* record );
     
+private:  
   void handleBoundSheet( BoundSheetRecord* record );
   void handleBOF( BOFRecord* record );
   void handleBoolErr( BoolErrRecord* record );
@@ -2695,7 +3198,11 @@ private:
   void handleColInfo( ColInfoRecord* record );
   void handleDateMode( DateModeRecord* record );
   void handleDimension( DimensionRecord* record );
+  void handleExternName( ExternNameRecord* record );
+  void handleExternSheet( ExternSheetRecord* record );
+  void handleFilepass( FilepassRecord* record );
   void handleFormat( FormatRecord* record );
+  void handleFormula( FormulaRecord* record );
   void handleFont( FontRecord* record );
   void handleFooter( FooterRecord* record );
   void handleHeader( HeaderRecord* record );
@@ -2705,6 +3212,7 @@ private:
   void handleMergedCells( MergedCellsRecord* record );
   void handleMulBlank( MulBlankRecord* record );
   void handleMulRK( MulRKRecord* record );
+  void handleName( NameRecord* record );
   void handleNumber( NumberRecord* record );
   void handlePalette( PaletteRecord* record );
   void handleRightMargin( RightMarginRecord* record );
@@ -2712,12 +3220,19 @@ private:
   void handleRK( RKRecord* record );
   void handleRow( RowRecord* record );
   void handleSST( SSTRecord* record );
+  void handleString( StringRecord* record );
+  void handleSupbook( SupbookRecord* record );
   void handleTopMargin( TopMarginRecord* record );
   void handleXF( XFRecord* record );    
   
   Color convertColor( unsigned colorIndex );
   FormatFont convertFont( unsigned fontIndex );
   Format convertFormat( unsigned xfIndex );
+  UString decodeFormula( unsigned row, unsigned col, 
+    const FormulaTokens& tokens, bool openDocumentFormat=false );
+  
+  void mergeTokens( UStringStack* stack, int count, char mergeChar );
+  void mergeTokens( UStringStack* stack, int count, const char* mergeString );
   
   // no copy or assign
   ExcelReader( const ExcelReader& );
@@ -2728,7 +3243,8 @@ private:
 };
 
 
-}; // namespace Sidewinder
+} // namespace Swinder
 
 
-#endif // SIDEWINDER_EXCEL_H
+
+#endif // SWINDER_EXCEL_H
