@@ -1,5 +1,3 @@
-/* $Id: mswriteexport.cc,v 1.18.2.1 2004/02/05 21:23:40 goutte Exp $ */
-
 /* This file is part of the KDE project
    Copyright (C) 2002-2003 Clarence Dang <dang@kde.org>
 
@@ -14,8 +12,8 @@
 
    You should have received a copy of the GNU Library General Public License
    Version 2 along with this library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <limits.h>
@@ -35,7 +33,7 @@
 #include <kdebug.h>
 #include <kgenericfactory.h>
 
-#include <koFilterChain.h>
+#include <KoFilterChain.h>
 #include <kowmfpaint.h>
 
 #include <KWEFStructures.h>
@@ -57,7 +55,7 @@ public:
 protected:
 	virtual void setupTranslations (void)
 	{
-		KGlobal::locale()->insertCatalogue ("kwordmswritefilter");
+		KGlobal::locale()->insertCatalogue ("kofficefilters");
 	}
 };
 
@@ -1907,7 +1905,7 @@ MSWriteExport::~MSWriteExport ()
 
 KoFilter::ConversionStatus MSWriteExport::convert (const QCString &from, const QCString &to)
 {
-	kdDebug (30509) << "MSWriteExport $Date: 2004/02/05 21:23:40 $ using LibMSWrite "
+	kdDebug (30509) << "MSWriteExport $Date: 2006-02-12 19:28:12 +0100 (dim, 12 fév 2006) $ using LibMSWrite "
 			  				<< MSWrite::Version << endl;
 
 	if (to != "application/x-mswrite" || from != "application/x-kword")
