@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "kivioglobal.h"
 #include "kivio_common.h"
@@ -43,9 +43,9 @@
 #include <qrect.h>
 #include <qbitmap.h>
 #include <kdebug.h>
-#include <koGlobal.h>
+#include <KoGlobal.h>
 #include <math.h>
-#include <kozoomhandler.h>
+#include <KoZoomHandler.h>
 
 /**
  * Default constructor
@@ -1411,7 +1411,6 @@ void KivioSMLStencil::setTextColor( QColor c )
     }
 }
 
-
 /**
  * Set the text font of this stencil
  */
@@ -1558,7 +1557,6 @@ KivioConnectorTarget *KivioSMLStencil::connectToTarget( KivioConnectorPoint *p,
  */
 void KivioSMLStencil::updateGeometry()
 {
-  KivioConnectorTarget *pTarget, *pOriginal;
   //kdDebug(43000) << "m_x = " << m_x << " m_y = " << m_y << endl;
 
   QWMatrix m;
@@ -1567,7 +1565,7 @@ void KivioSMLStencil::updateGeometry()
   m.rotate(m_rotation);
   m.translate(-m_w / 2.0, -m_h / 2.0);
 
-  pTarget = m_pConnectorTargets->first();
+  KivioConnectorTarget* pTarget = m_pConnectorTargets->first();
 
   while( pTarget )
   {
@@ -1604,6 +1602,23 @@ QFont KivioSMLStencil::textFont()
     return KoGlobal::defaultFont();
 }
 
+QColor KivioSMLStencil::textColor()
+{
+  KivioShape *pShape;
+
+  pShape = m_pShapeList->first();
+  while( pShape )
+  {
+    if( pShape->shapeData()->shapeType() == KivioShapeData::kstTextBox )
+    {
+      return pShape->shapeData()->textColor();
+    }
+
+    pShape = m_pShapeList->next();
+  }
+
+  return QColor(0, 0, 0);
+}
 
 /**
  * Sets the horizontal alignemnt of this stencil.
@@ -2093,6 +2108,22 @@ void KivioSMLStencil::addConnectorTarget(const KoPoint& p)
   m_pConnectorTargets->append(target);
 }
 
+void KivioSMLStencil::removeConnectorTarget(const KoPoint& pos)
+{
+  KivioConnectorTarget* target = m_pConnectorTargets->first();
+
+  while(target)
+  {
+    if(target->position() == pos)
+    {
+      m_pConnectorTargets->remove(target);
+      return;
+    }
+
+    target = m_pConnectorTargets->next();
+  }
+}
+
 bool KivioSMLStencil::hasTextBox() const
 {
   KivioShape* pShape = m_pShapeList->first();
@@ -2105,4 +2136,93 @@ bool KivioSMLStencil::hasTextBox() const
   }
 
   return false;
-}
\ No newline at end of file
+}
+
+QColor KivioSMLStencil::textColor(const QString& textBoxName)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape)
+    return textColor();
+
+  return shape->shapeData()->textColor();
+}
+
+void KivioSMLStencil::setTextColor(const QString& textBoxName, const QColor& color)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  // If we didn't find the shape set the color stencil wide
+  if(!shape) {
+    setTextColor(color);
+    return;
+  }
+
+  shape->shapeData()->setTextColor(color);
+}
+
+QFont KivioSMLStencil::textFont(const QString& textBoxName)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape)
+    return textFont();
+
+  return shape->shapeData()->textFont();
+}
+
+void KivioSMLStencil::setTextFont(const QString& textBoxName, const QFont& font)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape) {
+    setTextFont(font);
+    return;
+  }
+
+  shape->shapeData()->setTextFont(font);
+}
+
+int KivioSMLStencil::hTextAlign(const QString& textBoxName)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape)
+    return hTextAlign();
+
+  return shape->shapeData()->hTextAlign();
+}
+
+int KivioSMLStencil::vTextAlign(const QString& textBoxName)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape)
+    return vTextAlign();
+
+  return shape->shapeData()->vTextAlign();
+}
+
+void KivioSMLStencil::setHTextAlign(const QString& textBoxName, int align)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape) {
+    setHTextAlign(align);
+    return;
+  }
+
+  shape->shapeData()->setHTextAlign(align);
+}
+
+void KivioSMLStencil::setVTextAlign(const QString& textBoxName, int align)
+{
+  KivioShape* shape = locateShape(textBoxName);
+
+  if(!shape) {
+    setVTextAlign(align);
+    return;
+  }
+
+  shape->shapeData()->setVTextAlign(align);
+}
