@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qptrlist.h>
@@ -25,12 +25,14 @@
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <klocale.h>
+#include <ksimpleconfig.h>
 #include <kstandarddirs.h>
 
-#include <koDocument.h>
+#include <KoDocument.h>
 
 #include "contextstyle.h"
 #include "creationstrategy.h"
+#include "oasiscreationstrategy.h"
 #include "kformulacontainer.h"
 #include "kformuladocument.h"
 #include "sequenceelement.h"
@@ -39,17 +41,10 @@
 
 KFORMULA_NAMESPACE_BEGIN
 
-
 static const int CURRENT_SYNTAX_VERSION = 1;
 // Make sure an appropriate DTD is available in www/koffice/DTD if changing this value
 static const char * CURRENT_DTD_VERSION = "1.3";
 
-/**
- * The creation strategy that we are going to use.
- */
-static OrdinaryCreationStrategy creationStrategy;
-
-
 int FormulaList::compareItems( QPtrCollection::Item a, QPtrCollection::Item b )
 {
     double ya = static_cast<Container*>( a )->getDocumentY();
@@ -68,10 +63,10 @@ int FormulaList::compareItems( QPtrCollection::Item a, QPtrCollection::Item b )
 
 Document::Document( QObject *parent, const char *name,
                     const QStringList &/*args*/ )
-    : QObject( parent, name ), m_wrapper( 0 ), m_formula( 0 )
+    : QObject( parent, name ), m_wrapper( 0 ), m_formula( 0 ), creationStrategy( 0 )
 {
     m_contextStyle = new ContextStyle;
-    SequenceElement::setCreationStrategy( &creationStrategy );
+    setCreationStrategy( "Oasis" );
     formulae.setAutoDelete( false );
 }
 
@@ -127,8 +122,10 @@ int Document::formulaCount()
 }
 
 
-bool Document::loadXML( QDomDocument doc )
+bool Document::loadXML( const QDomDocument& doc )
 {
+    setCreationStrategy( "Ordinary" );
+        
     //clear();
     QDomElement root = doc.documentElement();
 
@@ -163,6 +160,14 @@ bool Document::loadXML( QDomDocument doc )
     return formulae.count() > 0;
 }
 
+bool Document::loadOasis( const QDomDocument& doc )
+{
+    // ### TODO: not finished!
+    setCreationStrategy( "Oasis" );
+    KFormula::Container* formula = newFormula( 0 );
+    return formula->loadMathML( doc, true );
+}
+
 bool Document::loadDocumentPart( QDomElement /*node*/ )
 {
     return true;
@@ -181,7 +186,7 @@ QDomDocument Document::saveXML()
 }
 
 
-QDomElement Document::saveDocumentPart( QDomDocument doc )
+QDomElement Document::saveDocumentPart( QDomDocument& doc )
 {
     QDomElement settings = doc.createElement( "FORMULASETTINGS" );
     return settings;
@@ -194,6 +199,41 @@ QDomDocument Document::createDomDocument()
                                           CURRENT_DTD_VERSION );
 }
 
+/**
+ * Create a MathML Dom Document, deprecates KFO Dom Document for internal layout
+ * TODO: Shouldn't this go to KoDocument ?
+ */
+QDomDocument Document::createMathMLDomDocument()
+{
+    QDomDocumentType dt = 
+        QDomImplementation().createDocumentType( "math",
+                                                 "-//W3C//DTD MathML 2.0//EN",
+                                                 "http://www.w3.org/TR/MathML2/dtd/mathml2.dtd");
+    QDomDocument doc( dt );
+    doc.insertBefore( doc.createProcessingInstruction( "xml", 
+                                                       "version=\"1.0\" encoding=\"UTF-8\"" ),
+                      doc.documentElement() );
+    return doc;
+}
+
+/**
+ * Set formula creation strategy: old KFormula or MathML/ODF.
+ * This tells which tags are valid during formula constructions
+ *
+ * @param strategy -- "Ordinary" for old Kformula, "Oasis" for MathML/ODF
+ */
+void Document::setCreationStrategy( QString strategy )
+{
+    if ( !creationStrategy || creationStrategy->type() != strategy ) {
+        delete creationStrategy;
+        if ( strategy == "Ordinary" )
+            creationStrategy = new OrdinaryCreationStrategy;
+        else
+            creationStrategy = new OasisCreationStrategy;
+        SequenceElement::setCreationStrategy( creationStrategy );
+    }
+}
+
 void Document::registerFormula( Container* f, int pos )
 {
     if ( ( pos > -1 ) &&
@@ -291,60 +331,7 @@ SymbolType Document::rightBracketChar()
 
 void Document::setEnabled( bool enabled )
 {
-    //kdDebug( DEBUGID ) << "Document::setEnabled " << enabled << endl;
-    m_wrapper->getAddNegThinSpaceAction()->setEnabled( enabled );
-    m_wrapper->getMakeGreekAction()->setEnabled( enabled );
-    m_wrapper->getAddGenericUpperAction()->setEnabled( enabled );
-    m_wrapper->getAddGenericLowerAction()->setEnabled( enabled );
-    m_wrapper->getAddOverlineAction()->setEnabled( enabled );
-    m_wrapper->getAddUnderlineAction()->setEnabled( enabled );
-    m_wrapper->getRemoveEnclosingAction()->setEnabled( enabled );
-    m_wrapper->getInsertSymbolAction()->setEnabled( enabled );
-    m_wrapper->getAddThinSpaceAction()->setEnabled( enabled );
-    m_wrapper->getAddMediumSpaceAction()->setEnabled( enabled );
-    m_wrapper->getAddThickSpaceAction()->setEnabled( enabled );
-    m_wrapper->getAddQuadSpaceAction()->setEnabled( enabled );
-    m_wrapper->getAddBracketAction()->setEnabled( enabled );
-    m_wrapper->getAddSBracketAction()->setEnabled( enabled );
-    m_wrapper->getAddCBracketAction()->setEnabled( enabled );
-    m_wrapper->getAddAbsAction()->setEnabled(enabled);
-    m_wrapper->getAddFractionAction()->setEnabled( enabled );
-    m_wrapper->getAddRootAction()->setEnabled( enabled );
-    m_wrapper->getAddSumAction()->setEnabled( enabled );
-    m_wrapper->getAddProductAction()->setEnabled( enabled );
-    m_wrapper->getAddIntegralAction()->setEnabled( enabled );
-    m_wrapper->getAddMatrixAction()->setEnabled( enabled );
-    m_wrapper->getAddOneByTwoMatrixAction()->setEnabled( enabled );
-    m_wrapper->getAddUpperLeftAction()->setEnabled( enabled );
-    m_wrapper->getAddLowerLeftAction()->setEnabled( enabled );
-    m_wrapper->getAddUpperRightAction()->setEnabled( enabled );
-    m_wrapper->getAddLowerRightAction()->setEnabled( enabled );
-    m_wrapper->getAppendColumnAction()->setEnabled( enabled );
-    m_wrapper->getInsertColumnAction()->setEnabled( enabled );
-    m_wrapper->getRemoveColumnAction()->setEnabled( enabled );
-    m_wrapper->getAppendRowAction()->setEnabled( enabled );
-    m_wrapper->getInsertRowAction()->setEnabled( enabled );
-    m_wrapper->getRemoveRowAction()->setEnabled( enabled );
-
-    if ( enabled ) {
-        m_wrapper->getAddGenericUpperAction()->
-            setShortcut( KShortcut( CTRL + Key_U ) );
-        m_wrapper->getAddGenericLowerAction()->
-            setShortcut( KShortcut( CTRL + Key_L ) );
-        m_wrapper->getRemoveEnclosingAction()->
-            setShortcut( KShortcut( CTRL + Key_R ) );
-        m_wrapper->getMakeGreekAction()->
-            setShortcut( KShortcut( CTRL + Key_G ) );
-        m_wrapper->getInsertSymbolAction()->
-            setShortcut( KShortcut( CTRL + Key_I ) );
-    }
-    else {
-        m_wrapper->getAddGenericUpperAction()->setShortcut( KShortcut() );
-        m_wrapper->getAddGenericLowerAction()->setShortcut( KShortcut() );
-        m_wrapper->getRemoveEnclosingAction()->setShortcut( KShortcut() );
-        m_wrapper->getMakeGreekAction()->setShortcut( KShortcut() );
-        m_wrapper->getInsertSymbolAction()->setShortcut( KShortcut() );
-    }
+    m_wrapper->setEnabled( enabled );
 }
 
 
@@ -371,11 +358,11 @@ void Document::updateConfig()
 }
 
 
-void Document::introduceWrapper( DocumentWrapper* wrapper )
+void Document::introduceWrapper( DocumentWrapper* wrapper, bool init )
 {
     m_wrapper = wrapper;
-    m_contextStyle->readConfig( wrapper->config() );
-    m_contextStyle->init();
+    m_contextStyle->readConfig( wrapper->config(), init );
+    m_contextStyle->init( init );
 }
 
 
@@ -392,6 +379,7 @@ DocumentWrapper::DocumentWrapper( KConfig* config,
 {
     if ( m_hasActions ) {
         createActions( collection );
+        enableMatrixActions( false );
     }
     setCommandStack( history );
 }
@@ -403,14 +391,33 @@ DocumentWrapper::~DocumentWrapper()
     if ( m_ownHistory ) {
         delete m_history;
     }
+
+    if ( m_hasActions )
+    {
+        m_config->setGroup("General");
+        m_config->writeEntry("syntaxHighlighting", m_syntaxHighlightingAction->isChecked() );
+    }
 }
 
 
-void DocumentWrapper::document( Document* document )
+void DocumentWrapper::document( Document* document, bool init )
 {
     m_document = document;
-    m_document->introduceWrapper( this );
+    m_document->introduceWrapper( this, init );
     initSymbolNamesAction();
+    m_config->setGroup("General");
+    if ( m_hasActions )
+    {
+        m_syntaxHighlightingAction->setChecked( m_config->readBoolEntry("syntaxHighlighting", true ) );
+        if ( !m_syntaxHighlightingAction->isChecked() )
+            toggleSyntaxHighlighting();
+    }
+    else if ( m_config->readBoolEntry("syntaxHighlighting", true ) )
+    {
+        m_document->m_contextStyle->setSyntaxHighlighting( true );
+        // Only to notify all views. We don't expect to get new values.
+        m_document->recalc();
+    }
 }
 
 
@@ -535,10 +542,12 @@ void DocumentWrapper::createActions( KActionCollection* collection )
                                       collection, "formula_addlowerright");
 
     m_addGenericUpperAction = new KAction(i18n("Add Upper Index"),
+                                      "gsup",
                                               /*CTRL + Key_U*/0,
                                       this, SLOT(addGenericUpperIndex()),
                                       collection, "formula_addupperindex");
     m_addGenericLowerAction = new KAction(i18n("Add Lower Index"),
+                                      "gsub",
                                       0,
                                       this, SLOT(addGenericLowerIndex()),
                                       collection, "formula_addlowerindex");
@@ -675,6 +684,17 @@ void DocumentWrapper::createActions( KActionCollection* collection )
                                          collection, "formula_fontfamily");
     m_fontFamily->setItems( ff );
     m_fontFamily->setEnabled( false );
+
+    QStringList et;
+    et.append( i18n( "Identifier" ) );
+    et.append( i18n( "Operator" ) );
+    et.append( i18n( "Number" ) );
+    et.append( i18n( "Text" ) );
+    m_tokenElement = new KSelectAction( i18n( "Token Type" ),
+                                     0, this, SLOT( tokenElement() ),
+                                     collection, "formula_tokenelement" );
+    m_tokenElement->setItems( et );
+//    m_tokenElements->setEnabled( true );
 }
 
 
@@ -1125,32 +1145,109 @@ void DocumentWrapper::fontFamily()
 }
 
 
+void DocumentWrapper::tokenElement()
+{
+    if ( hasFormula() ) {
+        int i = m_tokenElement->currentItem();
+        TokenElementType te = anyElement;
+        switch( i ) {
+        case 0: te = identifierElement; break;
+        case 1: te = operatorElement; break;
+        case 2: te = numberElement; break;
+        case 3: te = textElement; break;
+        }
+        TokenElementRequest r( te );
+        formula()->performRequest( &r );
+    }
+}
+
+
 void DocumentWrapper::initSymbolNamesAction()
 {
     if ( m_hasActions ) {
         const SymbolTable& st = m_document->m_contextStyle->symbolTable();
-
         QStringList names = st.allNames();
-        //QStringList i18nNames;
-        QValueList<QFont> fonts;
-        QMemArray<uchar> chars( names.count() );
+        QFont font( m_document->m_contextStyle->getFontStyle() );
+        QMemArray<QChar> chars( names.count() );
 
-        int i = 0;
+        uint i = 0;
         for ( QStringList::Iterator it = names.begin();
               it != names.end();
               ++it, ++i ) {
-            QChar ch = st.unicode( *it );
-            //i18nNames.push_back( i18n( ( *it ).latin1() ) );
-
-            fonts.append( st.font( ch ) );
-            chars[ i ] = st.character( ch );
+            chars[ i ] = st.unicode( *it );
         }
-        m_symbolNamesAction->setSymbols( names, fonts, chars );
+        m_symbolNamesAction->setSymbols( names, m_document->m_contextStyle->getMathFont(), chars );
         m_selectedName = names[0];
     }
 }
 
 
+void DocumentWrapper::setEnabled( bool enabled )
+{
+    kdDebug( DEBUGID ) << "DocumentWrapper::setEnabled " << enabled << endl;
+    getAddNegThinSpaceAction()->setEnabled( enabled );
+    getMakeGreekAction()->setEnabled( enabled );
+    getAddGenericUpperAction()->setEnabled( enabled );
+    getAddGenericLowerAction()->setEnabled( enabled );
+    getAddOverlineAction()->setEnabled( enabled );
+    getAddUnderlineAction()->setEnabled( enabled );
+    getRemoveEnclosingAction()->setEnabled( enabled );
+    getInsertSymbolAction()->setEnabled( enabled );
+    getAddThinSpaceAction()->setEnabled( enabled );
+    getAddMediumSpaceAction()->setEnabled( enabled );
+    getAddThickSpaceAction()->setEnabled( enabled );
+    getAddQuadSpaceAction()->setEnabled( enabled );
+    getAddBracketAction()->setEnabled( enabled );
+    getAddSBracketAction()->setEnabled( enabled );
+    getAddCBracketAction()->setEnabled( enabled );
+    getAddAbsAction()->setEnabled(enabled);
+    getAddFractionAction()->setEnabled( enabled );
+    getAddRootAction()->setEnabled( enabled );
+    getAddSumAction()->setEnabled( enabled );
+    getAddProductAction()->setEnabled( enabled );
+    getAddIntegralAction()->setEnabled( enabled );
+    getAddMatrixAction()->setEnabled( enabled );
+    getAddOneByTwoMatrixAction()->setEnabled( enabled );
+    getAddUpperLeftAction()->setEnabled( enabled );
+    getAddLowerLeftAction()->setEnabled( enabled );
+    getAddUpperRightAction()->setEnabled( enabled );
+    getAddLowerRightAction()->setEnabled( enabled );
+    
+    getAddGenericUpperAction()->setEnabled( enabled );
+    getAddGenericLowerAction()->setEnabled( enabled );
+    
+
+    if ( enabled ) {
+        getAddGenericUpperAction()->
+            setShortcut( KShortcut( CTRL + Key_U ) );
+        getAddGenericLowerAction()->
+            setShortcut( KShortcut( CTRL + Key_L ) );
+        getRemoveEnclosingAction()->
+            setShortcut( KShortcut( CTRL + Key_R ) );
+        getMakeGreekAction()->
+            setShortcut( KShortcut( CTRL + Key_G ) );
+        getInsertSymbolAction()->
+            setShortcut( KShortcut( CTRL + Key_I ) );
+    }
+    else {
+        getAddGenericUpperAction()->setShortcut( KShortcut() );
+        getAddGenericLowerAction()->setShortcut( KShortcut() );
+        getRemoveEnclosingAction()->setShortcut( KShortcut() );
+        getMakeGreekAction()->setShortcut( KShortcut() );
+        getInsertSymbolAction()->setShortcut( KShortcut() );
+    }
+}
+
+void DocumentWrapper::enableMatrixActions( bool b)
+{
+    getAppendColumnAction()->setEnabled( b );
+    getInsertColumnAction()->setEnabled( b );
+    getRemoveColumnAction()->setEnabled( b );
+    getAppendRowAction()->setEnabled( b );
+    getInsertRowAction()->setEnabled( b );
+    getRemoveRowAction()->setEnabled( b );
+}
+
 void DocumentWrapper::updateConfig()
 {
     m_syntaxHighlightingAction->
